#!/usr/bin/perl

BEGIN
{
    use File::Basename;
    my $dirprg = dirname($0);
    unshift @INC, "$dirprg/../../corelib";   
    unshift @INC, "$dirprg/../../lib";
}


use strict;
use warnings;
use General;
use GeneralBioinfo;
use ParamParser;
use Carp;

# svn propset svn:keywords "Id Rev" *.pl

# isoseq data (ccs+lima => oriented, anchored on 3' ==> polyA tail at the 3' end)
# extended to the 4 nt in order to facilitate gmap mapping

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s','out=s','prefix=s','bothends');

	$o_param->AssertFileExists('in');
	$o_param->AssertDefined('out');
	my $bothends = $o_param->IsDefined('bothends');

	my %h_seq = ();

	&FastaToHash($o_param->Get('in'),\%h_seq);

	my $f_out = $o_param->GetStreamOut('out');
	foreach my $id (sort keys(%h_seq))
	{
		my $seq = $h_seq{$id}{sequence};
		foreach my $nt ('A','T','G','C')
		{
			my $prevlen = 1;
			my $len     = 0;
			while( $prevlen != $len )
			{
				$prevlen = length($seq);
				$seq =~ s/$nt{10,}$//i;  			# removing polyX tail anchored at the sequence end
				$seq =~ s/$nt{10,}.{0,10}$//i;		# now add some tolerance (max 10nt)
				if ( defined($bothends) )
				{
					$seq =~ s/^$nt{10,}//i;  			# removing polyX head anchored at the sequence start
					$seq =~ s/^.{0,10}$nt{10,}//i;		# now add some tolerance (max 10nt)
				}
				$len = length($seq);
			}
		}
		printf $f_out ">$id len=%d\n",length($seq);
		print $f_out "$seq\n";
	}
	$f_out->close;
}

sub Usage
{
}

