# 
# 	sebastien.letort@toulouse.inra.fr
#	Created: May 09, 2007
#
# still under development ! use at your own risk.

package LipmError::ObjectException;

=pod

=head1 NAME

	LipmError::ObjectException - a class to define specific exception of Objects

=head1 SYNOPSIS

	throw LipmError::ObjectException( $class_name, KEYWORD, -text => $msg );
	throw LipmError::ObjectException( $obj, KEYWORD, -text => $msg );
	catch LipmError::ObjectException with {...}

=head1 DESCRIPTION

	With this class you can describe all type of exception encountered by object.

	- try to access to an attribute that doesn't exist
	- an attribute doesn't have a correct value

=head1 AUTHOR

Sebastien Letort : sebastien.letort@toulouse.inra.fr

=cut

use warnings;
use strict;

## this class inherit from LipmError.pm
use base qw(LipmError);

BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

# what use for that ?
use constant CAN_T_BE_INSTANCIED	=>	1;
use constant NO_BUILD	=>	2;
use constant UNKNOWN	=>	9;

# closure to make sure that %_h_msg will not be changed by other classes
# 	should i use constant ?
{

	my %_h_msg	=	(	'no-build'	=>	"object cannot be build.",
					''      	=>	"'no information transmitted' _can't guess what is wrong."
					);

	sub _GetMessage
	{
		my ( $keyword ) = @_;
		return $_h_msg{ $keyword };
	}
}

=head2 Object new

 Title        : new
 Usage        : my $o_err = new LipmError::ObjectException( $obj|$class_name, KEYWORD, -text => $msg);
 Prerequisite : it's better for an error to be catch !
 Object       : Constructor, initialize the object
 Returns      : the object.
 Args         :	none
 Globals      : none

=cut

sub new
{
	my $class = shift;
	my ($arg_un, $keyword) = (shift, shift);
	my %h_param = (
					@_
				);

	local $Error::Depth = $Error::Depth + 1;
	local $Error::Debug = 1;  # Enables storing of stacktrace

	# object and class
	my $tmp = ref( $arg_un );
	my ( $class_name, $obj ) = ($tmp eq '') ? ($arg_un, undef) : ($tmp, $arg_un);

	# keyword
	my $text = $h_param{ -msg };
	$text .= $class_name . ' ' . &_GetMessage($keyword) . "\n";


	my $o_err = $class->SUPER::new( -object => $obj,
									-class => $class_name,
									-text => $text,
									%h_param);

	return $o_err;
}

sub Explain
{
	my $self = shift;
	
	# Construct message
	my $msg = $self->GetMsg() . $self->text();

	return $msg;
}

1;
