#!/usr/bin/perl

#
# $Id: dbprot_extract_repet_domain.pl 1159 2016-02-06 11:26:56Z gouzy $
#

# Copyright INRA
#
# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# Ludovic.Legrand@toulouse.inra.fr
# Ludovic.Cottret@toulouse.inra.fr
# 

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=head1 NAME


=head1 SYNOPSIS


=head1 DESCRIPTION


=cut

BEGIN
{
        use File::Basename;
        my $dirprg=dirname($0);
        unshift @INC, "$dirprg/../../corelib";
        $VERSION = do { my @r = (q$Rev: 1159 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }
}

use strict;
use warnings;
use ParamParser;
use General;
use GeneralBioinfo;
use Carp;

my $NCPUS          = 8;
my $REPBASE        = "/db/license/repbase/repbase_aaSeq_cleaned_TE.fa";
my $BLASTPARAM     = " -evalue 1e-6 -max_target_seqs 2 -outfmt 6 -num_threads $NCPUS -seg yes ";
my $BLASTPPRG      = "blastp";
my $FASTAFILTERPRG = "lipm_fastafilter.pl";
my $M8TOM8PLUSPRG = "lipm_m8tom8plus.pl";
my $NRDBPRG        = "lipm_nrdb";
my $SCOV = "80";

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s','out=s','repbase=s','blastparam=s', 'no-clean', 'scov=s', 'blast_m8=s');
	$o_param->AssertFileExists('in');
	my $in  = $o_param->Get('in');
	$o_param->SetUnlessDefined('out',"$in.rep-domain");
	$o_param->SetUnlessDefined('repbase', $REPBASE);
	$o_param->AssertFileExists('repbase');

	$o_param->SetUnlessDefined('blastparam', $BLASTPARAM);
	$o_param->SetUnlessDefined('blastp_prg', $BLASTPPRG);
	$o_param->SetUnlessDefined('fastafilter_prg', $FASTAFILTERPRG);
	$o_param->SetUnlessDefined('m8tom8plus_prg', $M8TOM8PLUSPRG);
	$o_param->SetUnlessDefined('nrdb_prg', $NRDBPRG);
	$o_param->SetUnlessDefined('scov', $SCOV);
	

	my $blastpg       = $o_param->Get('blastp_prg');
	my $fastafilterpg = $o_param->Get('fastafilter_prg');
	my $m8tom8pluspg  = $o_param->Get('m8tom8plus_prg');

	my $nrdbprg       = $o_param->Get('nrdb_prg');

	&GetExePath($blastpg);
	&GetExePath($fastafilterpg);
	&GetExePath($m8tom8pluspg);

	my $out        = $o_param->Get('out');
	my $repbase    = $o_param->Get('repbase');
	my $blastparam = $o_param->Get('blastparam');
	my $outtmp     = "$in.vs.repeatdb";
	my $m8plustmp  = "$in.vs.repeatdb.m8plus";
	my $m8plustmpfilter = "$in.vs.repeatdb.m8plus.scovfilter";
	# Blast+ command line
	my $cde = "$blastpg -query $in -db $repbase $blastparam -out $outtmp";
	
	if ($o_param->IsDefined('blast_m8') && $o_param->AssertFileExists('blast_m8'))
	{
		$outtmp = $o_param->Get('blast_m8');
		print "Use m8 blast file $outtmp\n";
	}
	else
	{
		print "$cde\n";
		system($cde);
	}

	# no blastp hsp
	if (-z $outtmp)
	{
		my $f_out = &GetStreamOut($out);
		$f_out->close;
	}
	else
	{
		# Add qcov and scov
		$cde = "$m8tom8pluspg --m8 $outtmp --qfasta $in --sfasta $repbase --m8plus $m8plustmp";
		print "$cde\n";
		system($cde) && Carp::carp("ERROR\n");
		
		#&Filter M8plus file
		&M8PlusFilter($m8plustmp, $m8plustmpfilter, $o_param->Get('scov'));
		if (-z $m8plustmpfilter)
		{
			my $f_out = &GetStreamOut($out);
			$f_out->close;
		}
		else
		{
			$cde = "$fastafilterpg --in $in --out $out.raw0.gz --db_filename $m8plustmpfilter --db_column 1 --substr --cols \"7,8,1\"";
			print "$cde\n";
			system($cde);
			
			# suppression des seq redondantes
			$cde = "$nrdbprg -o $out.raw.gz $out.raw0.gz";
			print "$cde\n";
			system($cde);
			
			# suppression des | dans les ID
			my $f_in  = &GetStreamIn("$out.raw.gz");
			my $f_out = &GetStreamOut($out);
			my $ok=&FALSE;
			while(my $l=<$f_in>)
			{
				if ($l=~/^>/ )
				{
					$l =~ s,^>.+\|,>,;
					$l =~ s,\|\S*,,;
				}
				print $f_out $l;
			}
			$f_in->close;
			$f_out->close;
		}
	}
	
	if ( ! $o_param->IsDefined('no-clean') )
	{
		unlink("$outtmp") if ( (!$o_param->IsDefined('blast_m8')) && -e $outtmp);
		unlink("$m8plustmp") if (-e $m8plustmp);
		unlink("$m8plustmpfilter") if (-e $m8plustmpfilter);	
		unlink("$out.raw.gz")  if (-e "$out.raw.gz");
		unlink("$out.raw0.gz") if (-e "$out.raw0.gz");
	}

}



sub M8PlusFilter
{
	my ($m8plus, $m8plusfilter, $scov) = @_;
	
	my $f_in = &General::GetStreamIn($m8plus);
	my $f_out = &General::GetStreamOut($m8plusfilter);
	while (my $line = <$f_in>)
	{
		chomp($line);
		my @a_ = split(/\t/, $line);
		print $f_out "$line\n" if ($a_[13] >= $scov);
	}
	$f_in->close;
	$f_out->close;

	return;


}



sub Usage
{
print STDERR<<END
Prerequisite : blastp, lipm_fastafilter.pl,  lipm_nrdb and lipm_m8tom8plus.pl in the path
Extract proteins putatively related to transposable elements
$0
	--help
	--in           fastafile
[Optional]
	--out          fastafile [in.rep-domain]
	--repbase      filename  [$REPBASE] blast indexed repeat database
	--blastparam   string    [$BLASTPARAM]
    --scov         float     [$SCOV] Minimum required subject alignment percentage
    --blast_m8     filename  blastp result in m9 format
END
}

