#!/usr/bin/perl
#
# $Id $
#

# Copyright INRA

# Jerome.Gouzy@toulouse.inra.fr
# Erika.Sallet@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

=pod 

=head1 NAME

egn_consolidate_gff3.pl

=head1 SYNOPSIS


=head1 DESCRIPTION


=cut

BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../lib/";
use lib "$FindBin::Bin/../../corelib/"; 


use strict;
use General;
use GeneralBioinfo;
use ParamParser;
use Carp;
use Data::Dumper;

# Renomme les genes dans l'ordre d'arrivée. 
# Nom des gènes : seqid . "g" . $cpt
# $cpt est global a tous les fichiers

# si 'extra' est specifie, cree une version sans les epissages alternatifs, une version sans les ncRNAs, une sans les deux

my $DEFAULT_LEN_GENE_NUMBER    =  5;
my $DEFAULT_STEP               =  10;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'help','dirin=s','dirout=s', 'extra', 'len_gene_number=i','step=i');
	
	$o_param->AssertDirExists('dirin','dirout');
	$o_param->SetUnlessDefined('len_gene_number',    $DEFAULT_LEN_GENE_NUMBER);
	$o_param->SetUnlessDefined('step',    $DEFAULT_STEP);
	my $dirin  = $o_param->Get('dirin');
	my $dirout = $o_param->Get('dirout');
	my $step = $o_param->Get('step');
	
	my $len_gene_nb = $o_param->Get('len_gene_number');
	$len_gene_nb++ if ($step>1); 

	if ($dirin eq $dirout )
	{
		Carp::croak("dirin and dirout must be different\n");
	}
		
	
	my @a_gff3_raw = `find $dirin -maxdepth 1 -name "*.gff3" | sort `;

	# il faut faire plus sophistique vu que l'on a des Scaff1, Scaff10, Scaff2
	my @a_gff3 = sort SortByName @a_gff3_raw;

	my %h_corr = ();
	my %h_genetype = ();
	my $cpt = 1; # gene number
	# premier passage je construis la table de correspondance
	foreach my $gff (@a_gff3)
	{
		my $f_in = &GetStreamIn($gff);
		while(my $l=<$f_in>)
		{
			next if $l =~ /^#/;
			my %h_res=();
			&ParseGFF3Line(\$l,2,\%h_res);
			my ($type,$prefixid,$altcode) = &SplitIds($h_res{ID});

			if ( $h_res{type} eq 'gene' )
			{
				my $newid = sprintf("$h_res{seqid}g%0".$len_gene_nb."d", $cpt);
				$cpt+=$step;
				$h_corr{$prefixid} = $newid; 
				$h_corr{$newid} = $prefixid;	
			}
			elsif ( $h_res{type} eq 'mRNA' )
			{
				$h_genetype{$prefixid}='mRNA';
			}
		}
		$f_in->close;
	}

	# deuxieme passage j'edite
	# verifier l'unicite des ID
	my %h_ids = ();

	foreach my $gff (@a_gff3)
	{
		my $f_in = &GetStreamIn($gff);
		my ($dir,$f)=$gff=~/(.+)\/(.+)/;
		
		my $file      = $f;
		my $f_out_all = &GetStreamOut("$dirout/$file");
		
		my $f_out_noalt;
		my $f_out_noncrna;
		my $f_out_noaltnorncrna;
		
		if ($o_param->IsDefined('extra'))
		{
			$file = $f;
			$file =~ s/.gff3/.noalt.gff3/;
			$f_out_noalt = &GetStreamOut("$dirout/$file");
		
			$file = $f;
			$file =~ s/.gff3/.noncrna.gff3/;
			$f_out_noncrna = &GetStreamOut("$dirout/$file");
		
			$file = $f;
			$file =~ s/.gff3/.noncrna.noalt.gff3/;
			$f_out_noaltnorncrna = &GetStreamOut("$dirout/$file");
		}

		my $gff3_region  = "";
		my $gff3_version = 0;  # =1 if gff3 version line already read
		while(my $l=<$f_in>)
		{
			my %h_res = ();
			&ParseGFF3Line(\$l,2,\%h_res);
			if ($l =~ /^#/ )
			{
				if ($l =~ /^##gff-version/)
				{
					next if ($gff3_version == 1);
					$gff3_version = 1;
				}
				if ($l =~ /^##sequence-region/)
				{
					next if ($l eq $gff3_region);
					$gff3_region = $l;
				}
				
				print $f_out_all  $l;
				
				if ($o_param->IsDefined('extra'))
				{
					print $f_out_noalt "$l";
					print $f_out_noncrna "$l";
					print $f_out_noaltnorncrna "$l";
				}
				next;
			}
			
			my $seqid = $h_res{seqid};
			if ( ! defined($h_res{ID}) )
			{	
				print $f_out_all "$l";
				if ($o_param->IsDefined('extra'))
				{
					print $f_out_noalt "$l";
					print $f_out_noncrna "$l";
					print $f_out_noaltnorncrna "$l";	
				}
				next;
			}
			
			my $id = $h_res{ID};
			if ( defined($h_ids{$h_res{ID}}) )
			{
				$id = &EditIDs(\$l,$h_res{ID},\%h_ids);
			}
			$h_ids{$id}=1;
			my ($type,$prefixid,$altcode) = &SplitIds($id);
			my $newid = $h_corr{$prefixid};
			$l =~ s/:$prefixid/:$newid/g;
			$l =~ s/=$prefixid/=$newid/g;
			
			print $f_out_all "$l";
			
			next if (!$o_param->IsDefined('extra'));
			
			
			if ( ! defined($altcode) || $altcode eq 'a' )
			{
				print $f_out_noalt "$l";
			}	

			if ( defined($h_genetype{$prefixid}) && $h_genetype{$prefixid} eq 'mRNA')
			{
				print $f_out_noncrna "$l";
			}
			if ( defined($h_genetype{$prefixid}) && $h_genetype{$prefixid} eq 'mRNA' && ( !defined($altcode) || $altcode eq 'a' ) )
			{
				print $f_out_noaltnorncrna "$l";
			}
		}
		$f_in->close;
		$f_out_all->close;
		
		if ($o_param->IsDefined('extra'))
		{
			$f_out_noalt->close;
			$f_out_noncrna->close;
			$f_out_noaltnorncrna->close;
		}
	}
}

sub SortByName
{
my($fa)=$a=~/.+\/(.+)\.gff3/;
my($fb)=$b=~/.+\/(.+)\.gff3/;

my ($numa) = $fa =~ /(\d+)/;
my ($numb) = $fb =~ /(\d+)/;

	if ( $numa eq $numb ) # ne doit pas arriver sur les ID de scaff => cela veut dire que l'on a des chiffres dans la partie invariante du nom ==> je gere pas ==> sort alphanum
	{
		return $numa cmp $numb
	}
	else
	{
		return $numa <=> $numb
	}
}



sub EditIDs
{
my($rh_l,$id,$rh_ids) = @_;

	print STDERR "DUPLICATE ID:$id already defined $$rh_l";
	my $cpt=1;
	my $nid = $id;
	while(defined($$rh_ids{$nid}))
	{
		$nid = sprintf("$id.$cpt");
		$cpt++;
	}
	$$rh_l =~ s/$id/$nid/;
	print STDERR "DUPLICATE ID\t$id => $nid\n";
	return $nid;
}

=head2 fonction SplitIds

 Title        : SplitIds
 Usage        : 
 Prerequisite : none
 Function     : Extract information from the ID of a feature: type, prefix, code alternatif
 Returns      : ($type, $prefixid, $altcode). Per example CDS, PLHAL710p01.1, "?"
 Args         : $id       string   ID of an element. Example CDS:PLHAL710p01.15.1
 Access       : public
 Globals      : none 

=cut 
sub SplitIds
{
	my($id) = @_;
	# exon:PLHAL710p07.24.1             --> PLHAL710p07.24 (enleve fin)
	# exon:GMI11495-Rm2011G.a.4.1       --> GMI11495-Rm2011G.a.4 (enleve fin)
	# exon:PLHAL710p07.24.utr0          --> PLHAL710p07.24 (enleve fin)
    # exon:Mt0001_00004.1               --> Mt0001_00004 (enleve la fin)
    # [gene|mRNA|ncRNA]:scaffold51_69.2 --> scaffold51_69.2 (tout)
	# mRNA:GMI11495-Rm2011G.a.4         --> GMI11495-Rm2011G.a.4 (tout)
	my @a_id     = split(/[:\.]/, $id);  
	my $type     = $a_id[0];              # exon 
	my ($altcode) = $id =~ /([a-z])$/;   # last letter of an alternative gene 

	my $prefixid;
	if ($type eq "gene" || $type eq "mRNA" || $type eq "ncRNA" || $type eq "rRNA")
	{
		if ( $id =~ /ncrna/ ) # cas particulier des ncRNA avec exons
		{
			($prefixid) = $id =~ /.+:(.+)\.ncrna[^\.]+$/;
		}
		else
		{
			($prefixid) = $id =~ /.+:(.+)$/;
		}
	}
	else
	{
		($prefixid) = $id =~ /.+:(.+)\.[^\.]+$/;
	}
	return ($type, $prefixid, $altcode);
}

sub Usage	
{
print STDERR<<END
$0 
	--help
	--dirin   	directory
	--dirout  	directory
[Optional]
    --extra       If activate, create supplementary gff3 files: 
                    - one without ncRNA, 
                    - one without splicing variants 
                    - one without ncRNA and splicing variants
    --len_gene_number [5] Integer   Length of the number in the gene names SCA1g000001
END
}
