#!/usr/bin/perl
#
# $Id
#

# Copyright INRA

# Jerome.Gouzy@toulouse.inra.fr
# Erika.Sallet@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=pod

=head1 NAME

=cut

BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../lib/";
use lib "$FindBin::Bin/../../corelib/"; 


use strict;
use ParamParser;
use General;
use GeneralBioinfo;


MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',  \&Usage,  "in=s", "fof=s", 'ext=s', 'splitbystrand');

	$o_param->AssertFileExists('fof', 'in');

	if ( $o_param->IsDefined('splitbystrand') && !defined($ENV{EGNEP}) )
	{
		print STDERR "ERROR: The environment variable EGNEP has to be defined.\n" ;
		exit(1);
	}	
	
	my $splitbystrand = 0;
	$splitbystrand = 1 if($o_param->IsDefined('splitbystrand'));
	
	my $in = $o_param->Get('in');
	my $ext;
	if ($o_param->IsDefined('ext'))
	{
		$ext = $o_param->Get('ext');
	}
	else
	{
		($ext) = $in =~ /\.([^\.])\.gff3/i;
	}
	if ($ext eq "")
	{
		print STDERR "Impossible to infer the file extension. You should give one using --ext parameter\n"; 
		exit;
	}
	
	# Read the paths of the different genomic sequences
	my %h_seq_path;
	my $f_fof = $o_param->GetStreamIn('fof');
	while (my $line = <$f_fof>)
	{
		chomp($line);
		my ($seq) = $line =~ /.+\/([^\/]+)$/;
		$h_seq_path{$seq} = $line;
	}
	$f_fof->close;
	
	&Split($in, \%h_seq_path, $ext);

	&CompleteFileTree(\%h_seq_path, $ext, $splitbystrand);
}


=head2 procedure Split

    Title        : Split
    Usage        : &Split($file, $fof, $ext)
    Prerequisite : -
    Procedure    : split GFF3 $file per sequence name (column 1) and 
                   save each result file in the same directory than the sequence 
                   (sequence list in $fof)
    Args         : $o_param
                   $file        GFF3 file
                   $rh_seq_file Ref to a hash. Key=seqid, value=sequence file
                   $ext         String Extension of the result files 
   Prerequisite  : GFF3 file is sorted according to the sequence name
    Globals      : none
=cut
sub Split
{
	my ($file, $rh_seq_file, $ext) = @_;

	my $seqid = '';
	my $f_out = undef;

	my $f_in  = &GetStreamIn($file);
	while(my $l = <$f_in>)
	{
		chomp($l);
		next if ( $l =~ /^#/ );
		my @a_= split(/\t/, $l);
		
		if ( ! defined($f_out) || $a_[0] ne $seqid )
		{
			$f_out->close if defined ($f_out);
			$seqid = $a_[0];
			if (!defined $$rh_seq_file{$seqid})
			{
				print STDERR "Warning: The file of files does not contain the $seqid sequence. $seqid results are ignored\n";
				next;
			}
			my $seq = $$rh_seq_file{$seqid};
			$f_out = &GetStreamOut("$seq.$ext.gff3");
		}
		print $f_out "$l\n";
	}
	$f_in->close;
	$f_out->close if defined ($f_out);

	
	return;
}


=head2 procedure CompleteFileTree

    Title        : CompleteFileTree
    Usage        : CompleteFileTree ($rh_seq_f, $_ext, $splitbystrand)
    Prerequisite : -
    Procedure    : 
    Args         : $seq_fof      ref to an hash. Key=seqid, value=seq file
                   $ext          file extension
    Globals      : none
=cut
sub CompleteFileTree
{
	my ($rh_seq, $ext, $splitbystrand) = @_;
	
	foreach my $seqid (keys %$rh_seq)
	{
		my $seq = $$rh_seq{$seqid};
		if (! -e "$seq.$ext.gff3")
		{
			my $f_out = &GetStreamOut("$seq.$ext.gff3");
			$f_out->close;
		}
		if ($splitbystrand)
		{
			my $cmd = $ENV{EGNEP}."/bin/int/egn_splitbystrand.pl --gff3 $seq.$ext.gff3";
			system($cmd);
		}
	}
	
	return;
}

sub Usage
{

    print <<END
Usage: $0


Mandatory qualifiers:
 --in              gff3        Input gff3 file
 --fof             fof         File of sequence files.

prerequisite: gff3 file is sorted!!!!
[Optional]
 --ext             string      Extension of created files. If not fill, try to infer it by reading the input GFF3 file.
 --splitbystrand               Create to additional files, RV and FW, one by strand
END
;
}

