#!/usr/bin/perl

BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../lib/";
use lib "$FindBin::Bin/../../corelib/"; 


use strict;
use warnings;
use General;
use GeneralBioinfo;
use ParamParser;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'help','gff3=s','strict');
	$o_param->SetBehaviour('assert_empty_file_allowed', 1);
	$o_param->AssertFileExists('gff3');
	
	my $strict = 0;
	$strict = 1 if ($o_param->IsDefined('strict'));
	
	my $gff3 = $o_param->Get('gff3');
	my $root = $gff3;
	$root =~ s/.gff3$//i;
	
	my $f_fw = &GetStreamOut("$root.FW.gff3");
	my $f_rv = &GetStreamOut("$root.RV.gff3");
	
	my $f_in = $o_param->GetStreamIn('gff3');
	my %h_val=();
	
	while(my $l=<$f_in>)
	{
		&ParseGFF3Line(\$l,1,\%h_val);
		next if (! defined($h_val{strand}) );
		
		if ( ($h_val{strand} eq '+') ||  ($h_val{strand} eq '.' && !$strict))
		{
			print $f_fw $l;
		}
		if ( ($h_val{strand} eq '-') ||  ($h_val{strand} eq '.' && !$strict))
		{
			print $f_rv $l;
		}
	}
	
	$f_in->close;
	$f_fw->close;
	$f_rv->close;
}



sub Usage
{
print STDERR<<END
$0
	--help
	--gff3
[optional]
    --strict If defined, remove unoriented features
    --debug
END
}
