#!/usr/bin/perl

#
# $Id: get_LTRharvest.pl 1159 2016-02-06 11:26:56Z gouzy $
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

get_tRNAscanSE.pl

=head1 SYNOPSIS 

get_LTRharvest.pl  --sequence seq.fna  --outfile seq.ltrharvest..gff3  --cfg conf.cfg (can be the configuration file of EGNEP)

=head1 DESCRIPTION

run LTRharvest 

=cut


BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../corelib/"; 
use lib "$FindBin::Bin/../../lib/"; 
use strict;
use warnings;
use ParamParser;
use General;
use GeneralBioinfo;

my $DEBUG = 0;



MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG', \&Usage, "cfg=s", "sequence=s", "outfile=s", "workingdir=s", "no_clean", "debug");
	$o_param->Update('ENV', 'A'); 
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');
	$o_param->SetSubstitution('%i', \$ENV{EGNEP}) if ($ENV{'EGNEP'});
	$o_param->SetBehaviour('use_substitution_table');

	$DEBUG = 1 if ($o_param->IsDefined('debug'));
    # Test exist sequence and db
    $o_param->AssertFileExists('sequence');
    $o_param->AssertFullPath('sequence');
	$o_param->AssertDefined('outfile', 'prg_gt'); 
    my $sequence = $o_param->Get('sequence');
	my $outfile  = $o_param->Get('outfile');
	if (defined($outfile) && $outfile !~ /^\//)
    {
		my $pwd = `pwd`;
		chomp($pwd);
        $outfile = $pwd . "/" . $outfile;
        $o_param->Set('outfile', $outfile);
    }
	$o_param->AssertFullPath('outfile');
	$outfile = $o_param->Get('outfile');
	
	my ($outdir, $outfilename)  = $outfile =~ /(.+)\/(.+)/;
	$o_param->SetUnlessDefined('workingdir', $outdir);
	$o_param->AssertDirExists('workingdir');

	my $prg_gt     = $o_param->Get('prg_gt');
	my $workingdir = $o_param->Get('workingdir');
	my $cmd;

	# indexation
	my $indexfile = $workingdir."/$outfilename.index.$$.time";
	$cmd = "$prg_gt suffixerator -db $sequence -indexname $indexfile -tis -suf -lcp -des -ssp -sds -dna";
	print $cmd."\n" if $DEBUG;
	system($cmd) && Carp::croak "Failed during get_LTRharvest.pl >$cmd<\n";

	# ltrharvest
	my $stdoutfile = $workingdir."/$outfilename.stdout";
	$cmd = "$prg_gt ltrharvest -index $indexfile -seqids -out $outfile.fa -gff3 $outfile > $stdoutfile";
	print $cmd."\n" if $DEBUG;
	
	system($cmd) && Carp::croak "Failed during get_LTRharvest.pl $cmd<\n";
	
	# remove index files, and fa file
	if (! $o_param->IsDefined('no_clean'))
	{
		unlink ($indexfile.".des", $indexfile.".esq",$indexfile.".lcp",$indexfile.".llv");
		unlink ($indexfile.".md5", $indexfile.".prj",$indexfile.".sds",$indexfile.".ssp");
		unlink ($indexfile.".suf", $stdoutfile);
		#unlink("$outfile.fa");
	}

}

  
=item Usage

	$O_PARAM->Usage();
	Print the usage of the program

=cut

sub Usage
{
	print STDERR<<END
$0     
Launch LTRharvest

[Mandatory]
 --sequence             filename	fasta sequence file (gzipped file allowed)
 --outfile              filename    GFF3 result file.
 --cfg                  filename    Configuration file 

[Optional]
  --workingdir          dirname     [Default: outfile directory]  directory where create temporary files
  --no_clean                        If defined, dont remove temporary files
  --debug
  --help

END
}
