#!/usr/bin/perl

#
# $Id: get_tRNAscanSE.pl 1159 2016-02-06 11:26:56Z gouzy $
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

get_tRNAscanSE.pl

=head1 SYNOPSIS 

get_tRNAscanSE.pl  --sequence seq.fna  --outfile  seq.tRNAscan.gff3  --cfg conf.cfg (can be the configurartion file of EUGENE-PIPELINE)

=head1 DESCRIPTION

run trnascanSE and convert output in GFF3

=cut


BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../corelib/"; 
use lib "$FindBin::Bin/../../lib/"; 
use strict;
use warnings;
use ParamParser;
use General;
use GeneralBioinfo;

my $DEBUG = 0;



MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG', \&Usage, "cfg=s", "sequence=s", "outfile=s", "workingdir=s", "no_clean", "kingdom=s");
	$o_param->Update('ENV', 'A'); 
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');

	$o_param->SetUnlessDefined('kingdom', "eukaryote");
	$o_param->AssertAllowedValue('kingdom', "prokaryote", "eukaryote");
	my $env_var = ($o_param->Get('kingdom') eq "eukaryote") ? "EGNEP" : "EGNPP";

	$o_param->SetSubstitution('%i', \$ENV{$env_var}) if ($ENV{$env_var});
	$o_param->SetBehaviour('use_substitution_table');

    # Test the sequence exists and outfile is defined
    $o_param->AssertFileExists('sequence');
    $o_param->AssertFullPath('sequence');
	$o_param->AssertDefined('outfile'); 

    my $sequence = $o_param->Get('sequence');
	my $outfile  = $o_param->Get('outfile');
	if (defined($outfile) && $outfile !~ /^\//)
    {
		my $pwd = `pwd`;
		chomp($pwd);
        $outfile = $pwd . "/" . $outfile;
        $o_param->Set('outfile', $outfile);
    }
	$o_param->AssertFullPath('outfile');
	$outfile = $o_param->Get('outfile');
	
	my ($outdir, $outfilename)  = $outfile =~ /(.+)\/(.+)/;
	$o_param->SetUnlessDefined('workingdir', $outdir);
	$o_param->AssertDirExists('workingdir');

	$o_param->AssertDefined('prg_trnascanse', 'prg_convert_trnascan2gff3');

	chdir($o_param->Get('workingdir'));

	my $cmd;
	my $trnascansefile = $o_param->Get('workingdir')."/rawtrnascanse.".$$.time;

	$cmd =  $o_param->Get('prg_trnascanse');
	$cmd .= " -B " if ($o_param->Get('kingdom') eq "prokaryote");
	$cmd .= " -o $trnascansefile $sequence > /dev/null 2>&1";
	print $cmd."\n" if $DEBUG;
	system($cmd) && Carp::croak "Failed during trnascanSE >$cmd<\n";

	# convert to gff3
	$cmd = $o_param->Get('prg_convert_trnascan2gff3') . " --infile $trnascansefile --outfile $outfile";
	print $cmd."\n" if $DEBUG;
	system($cmd) && Carp::croak "Failed during >$cmd<\n";
	
	# remove raw trnascanse file
	unlink ($trnascansefile ) if (! $o_param->IsDefined('no_clean'));

}

  
=item Usage

	$O_PARAM->Usage();
	Print the usage of the program

=cut

sub Usage
{
	print STDERR<<END
$0     
Launch tRNAscanSE and convert result in GFF3

[Mandatory]
 --sequence             filename	fasta sequence file
 --outfile              filename    result file.
 --cfg                  filename    Configuration file 

[Optional]
  --kingdom             [eukaryote] eukaryote or prokaryote
  --workingdir          dirname     [Default: outfile directory]  directory where create temporary files
  --no_clean                        If defined, dont remove temporary files
  --help

END
}
