#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@inrae.fr
# Jerome.Gouzy@inrae.fr
# Thomas.Schiex@inrae.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;

my $DEF_PARENT = "EST_match";
my $DEF_CHILD  = "match_part";

my $DEBUG = 0;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s', 'out=s', 'parent=s', 'child=s', 'mindiff=i', 'histo', 'keep_unspliced', 'noclean', 'verbose', 'prev_method');

    $o_param->SetBehaviour('assert_empty_file_allowed', 1);
    $o_param->AssertFileExists('in');
    $o_param->AssertDefined('out');
    $o_param->SetUnlessDefined('parent', $DEF_PARENT); 
    $o_param->SetUnlessDefined('child',  $DEF_CHILD);
    $o_param->SetUnlessDefined('mindiff', 1);
    
    my $parent = $o_param->Get('parent');
    my $child  = $o_param->Get('child');
    my $out    = $o_param->Get('out');
    my $in     = $o_param->Get('in');
    my $mindiff = $o_param->Get('mindiff'); # NOT USED

    my %h_alignment; # key: intron (chr_start_end_strand) ; value: list the alignments name containing the intron
    my %h_introns;   # key: alignment name; value: list of its introns (chr_start_end_strand)
    
    my $intronnrfile            = $out.".intron.nr.gff3";
    my $sorted_intron_pair_file = "$intronnrfile.intersection.name.sort";
    
    #1) Je recupere la liste des introns distincts
    &ExtractNrIntrons($o_param, $in, $intronnrfile, $parent, $child, \%h_alignment, \%h_introns);
    
    # 2) Recherche des paires d'introns incompatibles
    # Creation du ficher des paires ordonnées d'introns incompatibles
    &GetIncompatibleIntronPairs($o_param, $intronnrfile, $sorted_intron_pair_file);
    
    if ($o_param->IsDefined('prev_method')) 
    {
        &Method1($o_param, $sorted_intron_pair_file, \%h_alignment, $parent, $out);
        exit;
    }
    

    my %h_inactive_introns;   # key= intronid. List the introns to ignored 
                              # because belonging only to alignments to remove
    my %h_alignment_priority; # key: alignment id, value: number of priority introns of this alignment
    my %h_alignment_toremove; # contient les ref des alignements a supprimer
    my %h_intron_already_view;# to flag the intron already view as more abondant than an other
    
    &PrintKeptTranscripts(\%h_introns, \%h_alignment_toremove) if ($DEBUG);
    
    # 3) Je traite paire apres paire (les paires sont triées dans l'ordre du nb de leur occurence)
    my $tokeep   = "";
    my $toremove = "";
    my $bestPriority1;
    my $bestPriority2;
    
    my $f_in = &GetStreamIn($sorted_intron_pair_file);
    while (my $line = <$f_in>)
    {
        chomp($line);
        my ($nb1, $intron1, $nb2, $intron2) = $line =~ /^(\d+)_(.+)\t(\d+)_(.+)$/;
		
        next if ( defined $h_inactive_introns{$intron1} || defined $h_inactive_introns{$intron2} );
        
        $bestPriority1 = &GetBestPriority($intron1, \%h_alignment, \%h_alignment_priority, \%h_alignment_toremove);
        $bestPriority2 = &GetBestPriority($intron2, \%h_alignment, \%h_alignment_priority, \%h_alignment_toremove);
        print "[DEBUG] >$line< Priorities: $bestPriority1 $bestPriority2\n" if ($DEBUG);
        $tokeep   = "";
        $toremove = "";
        
        if ($bestPriority1 > $bestPriority2)
        {
            $tokeep   = $intron1;
            $toremove = $intron2;
        }
        elsif ($bestPriority1 < $bestPriority2)
        {
            $tokeep   = $intron2;
            $toremove = $intron1;
        } 
        elsif ($nb1 ne $nb2)
        {
            # je ne regarde que si l'abondance est differente
            $tokeep   = $intron1;
            $toremove = $intron2;
        }
        # skip si aucun intron n'est prioritaire sur l'autre
        next if ( $tokeep eq "");
        
        if (!defined $h_intron_already_view{$tokeep})
        {
            # La premiere fois que l'on rencontre l'intron prioritaire,
            # On met a jour les priorites des alignements le contenant
            print "[DEBUG] Update aligment priorities containing $tokeep\n" if ($DEBUG);
            &UpdateAlignmentPriority($tokeep, \%h_alignment, \%h_alignment_priority);
            $h_intron_already_view{$tokeep} = 1;
        }
        &FlagAlignmentToRemove($toremove, \%h_alignment, \%h_alignment_toremove, \%h_introns, \%h_inactive_introns) ;
        &PrintKeptTranscripts(\%h_introns, \%h_alignment_toremove) if ($DEBUG);
    }
    $f_in->close;
    
    # Je supprime les alignements flagges
    &RemoveTranscript($in, $out, \%h_alignment_toremove, $parent);

    # copy  the second file if exists
    if ($o_param->IsDefined('keep_unspliced'))
    {
        my $unspliced = $in;
        $unspliced =~ s/\.gff3$//;
        $unspliced .= ".rm_unspliced_10000000.gff3";
        if (-e $unspliced)
        {
            my $new = $out;
            $new =~ s/\.gff3$//;
            $new .= ".rm_unspliced_10000000.gff3";
            my $cmd = "cp $unspliced $new";
            system($cmd);
        }
        else
        {
            print "warning: no unspliced file.$unspliced doesnt exist\n"; 
        }
    }
    
    if ($o_param->IsDefined('histo'))
    {
        my $cmd = "lipm_tab2histo.pl --in $out.ratio --out $out.ratio.histo --max 50 --step 1 --col 1";
        system($cmd);
        print "View histo of ratios :  $out.ratio.histo\n" if ($o_param->IsDefined('verbose'));
    }
 
    # FOR DEBUG: create a file listing the deleted alignements 
    if ($DEBUG)
    {
        my $f_out = &General::GetStreamOut($out.".listtranscrittoremove");
        foreach my $k (keys (%h_alignment_toremove))
        {
            print $f_out "$k\n";
        }
        $f_out->close;
    }
    
    if ($o_param->IsDefined('verbose'))
    {
        print "Nb alignments to remove: ". scalar(keys(%h_alignment_toremove))."/".scalar(keys(%h_introns))."\n";
    }
    
    if (!$o_param->IsDefined('noclean'))
    {
        unlink("$out.ratio.histo") if ($o_param->IsDefined('histo'));
        unlink("$out.ratio") ;
        unlink ("$intronnrfile", "$intronnrfile.intersection" , "$intronnrfile.intersection.name", "$intronnrfile.intersection.name.sort");	
        unlink($out.".listtranscrittoremove");
    }
}


# dans les alignements contenant cet intron, je cherche l'alignement 
# contenant le plus d'introns prioritaires. 
# Je retourne le nombre d'introns prioritaires
sub GetBestPriority
{
    my ($intronid, $rh_alignments, $rh_alignment_priority, $rh_alignment_toremove) = @_;
    
    my $max = 0;
    # get the alignments containing this intron
    my @a_alignments = split(',', $$rh_alignments{$intronid});

    foreach my $a (@a_alignments)
    {
        # je regarde un alignement ssi il n'est pas flaggé "toremove" 
        # et si il possède deja au moins un intron prioritaire
        next if ( defined $$rh_alignment_toremove{$a});
        next if (!defined $$rh_alignment_priority{$a});
        $max = $$rh_alignment_priority{$a}  if ($$rh_alignment_priority{$a} > $max);
    }
    return $max;
}


sub UpdateAlignmentPriority
{
    my ($intronid, $rh_alignments, $rh_alignment_priority) = @_;
    
    # get the list of all the alignments containing this intron
    my @a_a = split(',', $$rh_alignments{$intronid});
    foreach my $a (@a_a)
    {
        $$rh_alignment_priority{$a}++;
        print "-[DEBUG] Update the priority of alignement $a because of $intronid -> ".$$rh_alignment_priority{$a} ."\n" if ($DEBUG);
    }
    return;
}


sub RemoveTranscript
{
    my ($in, $out, $rh_alignment_toremove, $parent) = @_;
    
    my $f_in  = &GetStreamIn($in);
    my $f_out = &GetStreamOut($out);
    my $keep  = 1;
    my %h_val = ();
    
    while(my $l=<$f_in>)
    {
        chomp($l);
        &ParseGFF3Line(\$l,&TRUE,\%h_val);
        if ( $h_val{type} eq $parent ) # EST_match
        {
            if ( exists $$rh_alignment_toremove{$h_val{Name}} )
            {
                $keep = 0;
            }
            else
            {
                $keep = 1;
            }
        }
        print $f_out "$l\n" if ($keep);
    }
    $f_out->close;
    $f_in->close;
    
    return;
}
    
    

sub ExtractNrIntrons
{
    my ($o_param, $in, $out, $parent, $child, $rh_transcrit, $rh_introns) = @_;
    
    my %h_prevval = ();
    my %h_val     = ();
    
    my $global_intron_nb = 0;
    my %h_intron_nr = ();
    
    # 1) extraire les introns
    my $f_in  = &GetStreamIn($in);
    while(my $l=<$f_in>)
    {
        chomp($l);
        &ParseGFF3Line(\$l,&TRUE,\%h_val);
        if ( $h_val{type} eq $parent ) # EST_match
        {
            %h_prevval = ();
        }
        else # match_part
        {
            if ( $h_val{type} eq $child && defined($h_prevval{type}) && $h_prevval{type} eq $child )
            {
                # new intron
                $global_intron_nb++;
                my $start      = $h_prevval{end}+1;
                my $end        = $h_val{start}-1;
                my $len        = $end - $start + 1;
                my $strand     = $h_val{strand};
                my $chr        = $h_val{seqid};
                my $transcrit  =  $h_val{Name};
                my $key        = "$chr\_$start\_$end\_$strand";
                
                # Je compte l'intron
                $h_intron_nr{$key}++;
                
                # je mets à jour la liste des transcrits contenant cet intron
                $$rh_transcrit{$key}     .= "$transcrit,";
                
                # je met à jour la liste des introns de cet alignement
                $$rh_introns{$transcrit} .= "$key,";
            }
        }
        %h_prevval = ();
        %h_prevval = %h_val;
    }
    $f_in->close;

    # 2) je cree le fichier gff3 des introns sans redondance
    my $f_out = &GetStreamOut($out);
    foreach my $k (keys %h_intron_nr)
    {
        my ($c,$start, $end, $strand) =  $k =~ /^(.+)_(\d+)_(\d+)_(.+)$/;
        my $name = $h_intron_nr{$k}."_$k";
        print $f_out "$c\teugene\tintron\t$start\t$end\t.\t$strand\t.\tID=$name;Name=$name\n";
    }
    $f_out->close;
    
    if ($o_param->IsDefined('verbose'))
    {
        print "Nb of intron    = $global_intron_nb\n";
        print "Nb of intron NR =  " . scalar(keys(%h_intron_nr))."\n";
    }
    
    return;
}


#Recherche des paires d'intron incohérentes
# 1 Intersection avec lui meme pour rechercher les incoherences
# 2 Creation d'un fichier tabule de deux colonnes : intron le plus frequent puis intron le chevauchant avec une plus faible abondance
# 3 Tri de ce fichier par ordre décroissant d'abondance 
# TODO: virer les paires identiques tres tot!
sub GetIncompatibleIntronPairs
{ 
    my ($o_param, $intronnrfile, $sorted_intron_pair_file ) = @_;
    
    my $cmd;
    
    $cmd = "intersectBed -a $intronnrfile -b $intronnrfile -s -wa -wb > $intronnrfile.intersection";
    system($cmd);

    my %h_incoherent_couple; # key : intron name couple

    my $f_in  = &GetStreamIn( "$intronnrfile.intersection");
    my $f_out = &GetStreamOut("$intronnrfile.intersection.name");
    
    while (my $line = <$f_in>)
    {
        chomp($line);
        my @a_ = split(/\t/, $line);
        my %h_1;
        my %h_2;
        &GeneralBioinfo::ParseGFF3Line(\join("\t", @a_[0..8]), 1, \%h_1);
        &GeneralBioinfo::ParseGFF3Line(\join("\t", @a_[9..17]), 1, \%h_2);

        # intersection avec soi meme, je pass
        next if ($h_1{Name} eq $h_2{Name}) ;

        #  sort the two introns according to number of occurences (first most abundant)
        my @a_sort = sort MySort (($h_1{Name},$h_2{Name}));	
        my $couple = join ("\t", @a_sort); 

        # cette paire a deja ete vu, je pass!	
        next if (exists($h_incoherent_couple{$couple}));	

        print $f_out "$couple\n";
        $h_incoherent_couple{$couple} = 1;
    }
    $f_in ->close;
    $f_out->close;
    
    # je trie les lignes par nombre d'occurences des introns, colonne 1 puis colonne 2
    $cmd = "sort -s -k2,2nr $intronnrfile.intersection.name | sort -s -k1,1nr  > $sorted_intron_pair_file";
    system($cmd);
        
    print "Nb of incompatibilities = " . scalar(keys(%h_incoherent_couple)). "\n" if ($o_param->IsDefined('verbose'));
        
    if (!$o_param->IsDefined('noclean'))
    {
        unlink("$intronnrfile.intersection" , "$intronnrfile.intersection.name");	
    }
    
    return;
}

sub PrintKeptTranscripts
{
    my ($rh_introns, $rh_alignment_toremove) = @_;
    my $nbkeep = 0;
    print "----KEPT TRANSCRIPT---------------------\n";
    foreach my $k (keys %$rh_introns) {
        next if (defined $$rh_alignment_toremove{$k});
        $nbkeep++;
        print "$k ==> $$rh_introns{$k}\n";
    }
    print "===========================> $nbkeep KEPT\n";
    return;
}
        

# flag les alignements qui possèdent l'intron
sub FlagAlignmentToRemove
{
    my ($intron2remove, $rh_alignment, $rh_alignment_toremove, $rh_introns, $rh_inactive_introns) = @_;
    
    my %h_other_intron; # hash utilisé pour inactiver les introns qui n'appartiennent 
                        # QUE a des alignements a supprimer
    
    # je considere tous les alignements qui possèdent $intron2remove
    my $alignmentlist = $$rh_alignment{$intron2remove};
    $alignmentlist =~ s/,$//;
    my @a_alignment = split(",", $alignmentlist);
    foreach my $al (@a_alignment)
    {
        # the alignment al will be removed
        $$rh_alignment_toremove{$al} = 1; #.= "$intron2remove ";
        
        # je sauvegarde les introns de cet alignement
        foreach my $i (split (/,/, $$rh_introns{$al}))
        {
            $h_other_intron{$i} = 1;
        }
    }
    
    foreach my $i (keys (%h_other_intron))
    {
        next if (defined $$rh_inactive_introns{$i});
        
        # je regarde les autres alignements contenant cet intron
        my $a_list = $$rh_alignment{$i};
        $a_list =~ s/,$//;
        my $intron_to_desactivate = 1;
        foreach my $al (split(",", $a_list))
        {
            if (!defined $$rh_alignment_toremove{$al})
            {
                $intron_to_desactivate = 0 ;
                last;
            }
        }
        $$rh_inactive_introns{$i} = 1 if ($intron_to_desactivate);
        print "INACTIVATION OF $i\n" if ($DEBUG && $intron_to_desactivate);
    }
    return;
}


sub Method1
{
    my ($o_param, $sorted_intron_pair_file, $rh_transcrit, $parent, $out)  = @_;
    
    # 4) Je sauvegarde les references des introns qui sont incompatible avec un autre intron plus abondant
    my %h_toremove; # contient les ref des introns a supprimer
    
    my $f_o  = &GetStreamOut("$out.ratio");
    my $f_in = &GetStreamIn($sorted_intron_pair_file);
    while (my $line = <$f_in>)
    {
        chomp($line);
        my @a_ = split(/\t/, $line);

        my $name1 = $a_[0];
        my $name2 = $a_[1];
        my ($nb1) = $name1 =~ /^(\d+)_/;
        my ($nb2) = $name2 =~ /^(\d+)_/;
        print $f_o ($nb1/$nb2)."\n";
		
        # je ne regarde que si l'abondance est differente
        next if ($nb1 eq $nb2 ) ; # || ( abs($nb1 - $nb2) <= $mindiff) ; #|| 1;

        # je verifie que l'intron majoritaire n'a pas ete precedemment supprime !!
        next if (exists($h_toremove{$name1}));
        
        # le deuxieme est moins present que le premier, je le supprime
        $h_toremove{$name2}++;
    }
    $f_in->close;
    $f_o->close;

    # 5 ) look for mrna containing intron to remove
    my %h_transcrit_toremove;
    foreach my $intron (keys (%h_toremove))
    {
        my $key = $intron;
        $key =~ s/^\d+_//;
        my $list = $$rh_transcrit{$key};
        $list =~ s/,$//;
        my @a_list = split(",",$list);
        foreach my $t (@a_list)
        {
            $h_transcrit_toremove{$t}  .= "$intron ";
        }
    }

    # 6  Je supprime les alignements si le transcrit a un intron incoherent
    $f_in  = $o_param->GetStreamIn('in');
    my $f_out = $o_param->GetStreamOut('out');
    my $keep = 1;
    my %h_val;
    while(my $l=<$f_in>)
    {
        chomp($l);
        &ParseGFF3Line(\$l,&TRUE,\%h_val);
        if ( $h_val{type} eq $parent ) # EST_match
        {
            if ( exists $h_transcrit_toremove{$h_val{Name}} )
            {
                $keep = 0;
            }
            else
            {
                $keep = 1;
            }
        }
        print $f_out "$l\n" if ($keep);
    }
    $f_out->close;
    $f_in->close;
    
    return;
}



sub MySort
{
    my ($occa, $namea ) = $a =~ /^(\d+)_(.+)$/;
    my ($occb, $nameb) = $b =~ /^(\d+)_(.+)$/;
    if ($occa != $occb)
    {
	return ($b <=> $a)
    }
    else
    {
	return $a cmp $b;
    }
   
}


sub Usage
{
    print STDERR<<END
Filter intron incoherences in EST_match/match_part gff3 file.
If incoherence, keep the introns with the most occurences. 
All the alignments of a transcript with at least one  intron incoherence are removed.
If ---keep_unspliced and input file .rm_unspliced_10000000.gff3 exists, copy it to out .rm_unspliced_10000000.gff3
$0
Mandatory
   --in     filename    gff3 formatted file (has to end by 'gff3') 
   --out    filename    file name           (has to end by 'gff3') 
Optional
   --keep_unspliced
   --verbose            To print count information
   --parent  string     default="EST_match"
   --child   string     default="match_part"
   --histo
   --noclean
END
}
