#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;
use Data::Dumper;

my $MAXLEN = 10000;
my $MARGIN = 0;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'lof=s','out=s','seqlen=s','seqname=s','stranded','maxlen=i','tmpdir=s','margin=i');

	$o_param->AssertDefined('out');
	$o_param->SetUnlessDefined('maxlen',$MAXLEN);
	$o_param->SetUnlessDefined('margin',$MARGIN);

	my %h_options=undef;
	if ( $o_param->IsDefined('tmpdir') )
	{
		$o_param->AssertDirExists('tmpdir');
		$h_options{tmpdir}=$o_param->Get('tmpdir');
	}

	my $stranded = $o_param->IsDefined('stranded');
	my $maxlen = $o_param->Get('maxlen');
	my $margin = $o_param->Get('margin');
	my $seqlen = $o_param->GetIfDefined('seqlen');
	my $seqname = $o_param->GetIfDefined('seqname');
	my $out = $o_param->Get('out');

	my @a_cat=();
	my %h_hit=();

	my $ra_gff3 = $o_param->GetListOfValidFiles('lof');

    my $outraw      = "$out.raw.$$.gz";
    my $f_outraw = &General::GetStreamOut($outraw);
	# ca serait peut etre plus rapide grep -h -v Parent= | grep -v ^# > file.gff3
    foreach my $f (@$ra_gff3)
    {
        my $f_in = &General::GetStreamIn($f);
        while (my $line = <$f_in>)
        {
            chomp $line;
            next if $line =~ /^#/;
            next if $line =~ /Parent=/;
			if ( $stranded )
			{
				&ParseGFF3Line(\$line,0,\%h_hit);
				next if ( $h_hit{len} > $maxlen ); # trop risque sinon, on ne cherche pas a les avoir tous (c'est pour construire des regions non interg)
			}
			if ( $margin ) # lorsque l'on veut proteger une zone il faut prendre un peu de marge sinon on risque de louper le start ou le stop
			{
				my @a_ = split("\t",$line);
				$a_[3] -= $margin;
				$a_[3] = 1 if ( $a_[3] < 1 );
				$a_[4] += $margin;
				$a_[4] = $seqlen if ( $a_[4] > $seqlen );
				$line = join("\t",@a_);
			}
			printf $f_outraw "$line\n";
        }
        $f_in->close;
    }
	$f_outraw->close;

    my $f_sort_cat_gff3 = &General::GetStreamOut($out);
    my $f_outraw        = &General::GetStreamIn($outraw);
	if (  ! $stranded )
	{
		print $f_sort_cat_gff3 "##gff-version 3\n";
		print $f_sort_cat_gff3 "##sequence-region $seqname 1 $seqlen\n" if ( defined($seqname) && defined($seqlen) );
	}
	my $r_options=(%h_options)?\%h_options:undef;
	&General::UnixSort($f_outraw,$f_sort_cat_gff3, "4n",$r_options);
    $f_sort_cat_gff3->close;
	unlink($outraw);
}


sub Usage
{
    print STDERR<<END
Extract Parent features from a list of gff3 files and build a sorted gff3 file
Mandatory
   --lof     listoffiles gff3 formatted files
   --out     filename    gff3 formatted file
Optional
   --seqlen  integer     sequence length
   --seqname string      sequence ID
   --tmpdir  dirname     used for the unix sort
Dev
   --stranded
   --maxlen  integer     [$MAXLEN]
Warning: works for gff3 files corresponding to a single sequence 
END
}
