#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;
use Data::Dumper;

my $MINLEN = 25;

my $spliced_ext   = "";
my $unspliced_ext = ".rm_unspliced_10000000";
MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s', 'ext=s', 'splitbystrand');
    $o_param->SetBehaviour('assert_empty_file_allowed', 1);
    
    $o_param->AssertFileExists('in');
    $o_param->AssertDefined('ext'); 
    
    my $in    = $o_param->Get('in'); 
    my ($out) = $in =~ /(.+)\.[^\.]+\.gff3$/i;
    $out.= ".".$o_param->Get('ext');
    
    my $spliced   = $out.$spliced_ext.".gff3";
    my $unspliced = $out.$unspliced_ext.".gff3";
    

    
    my $match;
    my %h_res=();
    my @a_hit=();
    my $f_in = $o_param->GetStreamIn('in');
    my $f_out_spliced    = &GetStreamOut( $spliced);
    my $f_out_unspliced  = &GetStreamOut( $unspliced);
    
    while(my $l=<$f_in>)
    {
	chomp($l);
	&ParseGFF3Line(\$l,&TRUE,\%h_res);
	if ( $h_res{type} eq 'EST_match')
	{
	    &Split( $f_out_spliced, $f_out_unspliced,$match,\@a_hit) if ($match ne "");
	    @a_hit=();
	    $match=$l;
	}
	else
	{
	    push(@a_hit,$l);
	}
	
    }
    &Split( $f_out_spliced, $f_out_unspliced,$match,\@a_hit) if ($match ne "");
    
    $f_in->close;
    $f_out_spliced ->close;
    $f_out_unspliced ->close;
    if ($o_param->IsDefined('splitbystrand'))
    {
	my $cmd = $ENV{EGNEP}."/bin/int/egn_splitbystrand.pl --gff3 $spliced";
	system($cmd);
	$cmd = $ENV{EGNEP}."/bin/int/egn_splitbystrand.pl --gff3 $unspliced";
	system($cmd);
    }
}


sub Split
{
    my($f_spliced, $f_unspliced, $match, $ra_hit)=@_;

    my $f_ ;
    if ( scalar(@$ra_hit) > 1 ) # il y a au moins deux hits
    {
	$f_ = $f_spliced;
    }
    else
    {
	$f_ = $f_unspliced;
    }
    
    print $f_ "$match\n";
    for(my $i=0;$i<scalar(@$ra_hit);$i++)
    {
	print $f_ "$$ra_hit[$i]\n";
    }
}



sub Usage
{
    print STDERR<<END
Split spliced and unspliced ESTs alignments. 
$0
Mandatory
   --in     filename    gff3 formatted file (match/match_part)
   --ext    out extension (ex est6)
Optional
   --splitbystrand
END
}
