#
# $Id: AnnotatedSeqFileReader.pm 1288 2013-10-21 08:39:57Z manu $
#

package AnnotatedSeqFileReader;

use strict;
use warnings;

use base qw( LipmObject );

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

AnnotatedSeqReader - Read an annotation file, maybe a sequence file, and load data to a companion AnnotatedSeq object

=head1 SYNOPSIS

	See AnnotatedSeq

=head1 DESCRIPTION

Read an annotation file, maybe a sequence file, and load data to a companion AnnotatedSeq object
Real interesting stuff is in the DERIVED classes
All the classes from the AnnotatedSeqReader hierarchy use PRIVATE of PROTECTED fields of AnnotatedSeq
(in C++ we would have to declare those classes as FRIENDS of AnnotatedSeq)
                     
NOTE - This object manages only ONE sequence file (FASTA, not MULTIFASTA) and ONE annotation file

AnnotatedSeqReader has ONLY ONE data member: __o_annot, the AnnotatedSeq object in which it is included

=cut

use Carp;
use General;
use LipmError::ParameterException;
use General;
use Error qw(:try);
use Locus;
use Locus::Gene;
use Locus::Mrna;
use Locus::Ncrna;
use Locus::Cds;
use Locus::Repeat;
use Data::Dumper;

# a mettre dans un dumper xml
use FileHandle;
use XML::Twig;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1288 $ =~ /\d+/g); $r[0]};
}


=head2 procedure _Init

 Title        : Constructor
 Usage        : Use child classes (AnnotatedSeqFile::GenBank, AnnotatedSeqFile::GFF3, etc.)
 Function     : Initialize the object
 Args         : $o_annot The AnnotatedSeq object inside which we live
 Globals      : none

=cut

sub _Init
{
	my $self = shift;
	my ($o_annot) = @_;
	
	if ( ref($o_annot) ne 'AnnotatedSeq' )
	{
		throw LipmError::ParameterException( 'o_annot', 'type', -text => 'should be an AnnotatedSeq object' );
	}
	$self -> { _o_annot } = $o_annot;
	return $self;
}

=head2 Abstract or default procedures and functions

 Title        : GetFormat, Load, DetectChromosome, LoadHeaders
 Usage        : Virtual methods
 Function     : Ensure that the class AnnotatedSeqFile is not directly instantiated
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub GetFormat()
{
	throw LipmError::ObjectException( 'AnnotatedSeqFileReader', 'abstract method' );
}
sub Load
{
	my ($self) = @_;
	throw LipmError::ObjectException( 'AnnotatedSeqFileReader', 'abstract method' );
}
sub LoadGenomicSequence
{
	my ($self) = @_;
	throw LipmError::ObjectException( 'AnnotatedSeqFileReader', 'abstract method' );
}
sub DetectChromosome
{
	my ($self) = @_;
	throw LipmError::ObjectException( 'AnnotatedSeqFileReader', 'abstract method' );
}
sub LoadHeaders{}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
