#
# $Id: AnnotatedSeqFileWriter.pm 1288 2013-10-21 08:39:57Z manu $
#

package AnnotatedSeqFileWriter;

use strict;
use warnings;

use base qw( LipmObject );

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

AnnotatedSeqWriter - Dump to a file data stored in a companion AnnotatedSeq object

=head1 SYNOPSIS

 	See AnnotatedSeq

=head1 DESCRIPTION

Read data stored by a companion AnnotatedSeq object and dump them to a file.
Real interesting stuff is in the DERIVED classes
All the classes from the AnnotatedSeqWriter hierarchy use PRIVATE of PROTECTED fields of AnnotatedSeq
(in C++ we would have to declare those classes as FRIENDS of AnnotatedSeq)
                     
DO NOT use this class, use a derived class instead

=cut

use Carp;
use LipmError::ObjectException;
use LipmObject;

use base qw( LipmObject );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1288 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Args         : $o_annot An AnnotatedSeqfile object

=cut

sub _Init
{
	my $self = shift;
	my ($o_annot) = @_;

	$self->SUPER::_Init();
	$self -> { _o_annot } = $o_annot;
}

=head2 procedure Dump

 Title        : Dump
 Usage        : $o_annot_writer -> DumpFastaDb ( %h_param )
 Procedure    : Dump the gene objects to some formatted file
 Args         : -append = 0/1 (default 0) If 1, the file opened in append mode
                 The other (keys, values) depend on the class (on the format)

=cut
sub Dump {
	my $self = shift;
	throw LipmError::ObjectException( 'AnnotatedSeqFileWriter', 'abstract method' );
}
sub GetFormat()
{
	throw LipmError::ObjectException( 'AnnotatedSeqFileWriter', 'abstract method' );
}

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut


1;
