#
# $Id: Ncrna.pm 1336 2013-12-04 13:35:33Z manu $
#

package Locus::Ncrna;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

Locus::Ncrna - describes a ncrna

=head1 SYNOPSIS

=head1 DESCRIPTION

 A Ncna extends the Locus::Rna class
 The parent may be a Locus::Gene object or a AnnotatedSeq object

=cut

use strict;
use warnings;
use Carp;
use Bio::Seq;
use Data::Dumper;

use Locus;

use base qw( Locus::Rna );

sub GetDefaultSubType { return 'ncRNA' };

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1336 $ =~ /\d+/g); $r[0]};
}

=head2 procedure SetParent

 Title        : SetParent
 Usage        : $o_ncrna->SetParent( $o_locus );
 Procedure    : Set the parent
 Prerequisite : The parent should not already set, $o_locus should be:
                    1/ An AnnotatedSeq object
                    2/ OR a Locus::Gene object
 
 Args         : $o_locus : The parent, an AnnotatedSeq object or a Locus::Gene object
 Globals      : none

=cut
sub SetParent
{
	my $self = shift;
	my ($o_locus) = @_;
	
	if ( ref($o_locus) ne 'Locus::Gene' && ref($o_locus) ne 'AnnotatedSeq' )
	{
		throw LipmError::ParameterException( 'o_parent', 'type', -text => 'should be a Locus::Gene OR an AnnotatedSeq object' );
	}

	$self->SUPER::SetParent( $o_locus );
	
	my $nc_id = $o_locus -> GetNarcisseId();
	$nc_id   .= 'n' . $self -> GetCount();
	$self -> { __narcisse_id } = $nc_id;
}

=head2 procedure AddTo

=cut
sub AddTo { 
	my $self = shift;
	my ($o_annot) = @_;
	$o_annot->AddGene($self);
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


