#
# $Id: Utr.pm 1288 2013-10-21 08:39:57Z manu $
#

package Locus::Utr;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

Locus::Utr - describes an UTR

=head1 SYNOPSIS

=head1 DESCRIPTION

 An Utr extends the Locus class. An Utr contains UtrRegions and is managed like the Exon
 The parent of an Utr is a Locus::Mrna object
 An Utr should not have a locus_tag nor gene_name attribute, and does not have any NarcisseId
 It should NOT be printed, only the UtrRegions will be printed
 Thus, an Utr is ONLY a container for UtrRegions

=cut

use strict;
use warnings;
use Carp;
use Bio::Seq;
use Data::Dumper;
use General;
use Locus;

use base qw( Locus );

sub GetDefaultSubType { return 'UTR' };

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1288 $ =~ /\d+/g); $r[0]};
}

=head2 procedures Select,Deselect

 See Locus.pm for the generic documentation
 we do not change the status of the children (see UtrRegion::IsSelected for the details)

=cut
sub Select
{
	my $self = shift;
	delete $self -> { __deselected };
}
sub Deselect
{
	my $self   = shift;
	$self -> { __deselected } = &TRUE;
}

=head2 procedure SetParent

 Title        : SetParent
 Usage        : $o_gene->SetParent( $o_mrna );
 Procedure    : Set the parent
 Prerequisite : The parent should not already set
 
 Args         : $o_annot : The parent, should be a Locus::Mrna object
 Globals      : none

=cut
sub SetParent
{
	my $self = shift;
	my ($o_mrna) = @_;
	
	if ( ref($o_mrna) ne 'Locus::Mrna' )
	{
		throw LipmError::ParameterException( 'o_parent', 'type', -text => 'should be a Locus::Mrna object' );
	}
	
	# s1g1t1 ==> s1g1Utr1
	my $nc_id = $o_mrna -> GetNarcisseId();
	$nc_id    =~ s/t(\d+)$/Utr$1/;
	$self -> { __narcisse_id } = $nc_id;
	
	$self->SUPER::SetParent( $o_mrna );
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


