#!/usr/bin/perl

#
#       emmanuel.courcelle@toulouse.inra.fr
#       $Id$
#
#       test file for EnzymeValidation.pm

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../../../corelib";
use lib "$FindBin::Bin/../../../lib/";
use lib "$FindBin::Bin/../../../lib/BioFileConverter";

# Lorsque la classe est finalisee, supprimez le no_plan
#use Test::More qw(no_plan);
use Test::More;
#plan( tests =>17 );
use Test::MockModule;

use Locus;
use Locus::MiscRegion;

# mocking two functions of the module AnnotatedSeq
my $o_mock = new Test::MockModule('AnnotatedSeq');
$o_mock->mock('GetGenomicSequence',\&GetGenSeq);
#$o_mock->mock('__GetEnzymeDat',\&GetEnzymeDat);

MAIN:
{
	&TestDependencies();
#	&TestConstants();
	&TestConstructors();
	&TestUtils();
	&TestGetSequence();
}

sub TestDependencies
{
	diag( "---- Test Dependencies ----\n" );
	
	use_ok 'LipmObject';
	use_ok 'General';
	print "\n";
}

sub TestConstructors
{
	diag( "---- Test Constructors ----\n");
	
	# We test a MiscRegion constructor, because Locus is an abstract class !
	my $o_locus = New Locus::MiscRegion( start=>1000,end=>1500,strand=>1 );
	isa_ok( $o_locus, "Locus", "Testing a constructor");
	print "\n";
}

sub TestUtils
{
	diag( "---- Test Utility functions\n" );
	
	my $seq = &GenerateSeq();
	my $vseq= &Vseq($seq);

	#print "$seq\n";
	#print "$vseq\n";
	
	my $r_seq = \$seq;
	is ( &Locus::UtilRevComp('ACGTAACCGGTTAAAC'),'GTTTAACCGGTTACGT' , ' Testing UtilRevComp');
	is ( &Locus::UtilGetSeqPre($r_seq,34,10),'TAAAACCCCG','1. Testing UtilGetSeqPre');
	is ( &Locus::UtilGetSeqPre($r_seq,10,10),' ACGTAACCG','2. Testing UtilGetSeqPre');
	is ( &Locus::UtilGetSeqPre($r_seq,11,10),'ACGTAACCGG','3. Testing UtilGetSeqPre');
	is ( &Locus::UtilGetSeqPre($r_seq,5,10), '      ACGT','4. Testing UtilGetSeqPre');
	is ( &Locus::UtilGetSeqPost($r_seq,34,10), 'GGTTTTAAAA','1. Testing UtilGetSeqPost');
	is ( &Locus::UtilGetSeqPost($r_seq,335,10),'GAACCGGACG','2. Testing UtilGetSeqPost');
	is ( &Locus::UtilGetSeqPost($r_seq,336,10),'AACCGGACG ','3. Testing UtilGetSeqPost');
	is ( &Locus::UtilGetSeqPost($r_seq,339,10),'CGGACG    ','4. Testing UtilGetSeqPost');
	
	print "\n";
}

sub TestGetSequence
{
	diag ( "---- Test GetSequence and GetSequenceWithEnvironment\n" );
	
	my $o_annot = New AnnotatedSeq(kingdom=>'bacteria',species=>'b');
	my $o_locus = New Locus::MiscRegion( start=>10,end=>20,strand=>1);
	$o_locus->SetParent($o_annot);
	&is ( $o_locus->GetSequence(),'GTTAAACCCGG' , ' 1. S+ Testing GetSequence');
	&is ( $o_locus->GetSequenceWithEnvironment(5),'AACCGGTTAAACCCGGGTTTA',' 1. S+ M Testing GetSequenceWithEnvironment' );
	&is ( $o_locus->GetSequenceWithEnvironment(10),' ACGTAACCGGTTAAACCCGGGTTTAAAACC','2. S+ B Testing GetSequenceWithEnvironment' );
	&is ( $o_locus->GetSequenceWithEnvironment(15),'      ACGTAACCGGTTAAACCCGGGTTTAAAACCCCGGG','3. S+ B Testing GetSequenceWithEnvironment' );
	print "\n";
	
	my $o_locus1= New Locus::MiscRegion( start=>321,end=>330,strand=>1);
	$o_locus1->SetParent($o_annot);
	&is ( $o_locus1->GetSequence(),'CCCGGGGAAA' , ' 2. S+ Testing GetSequence');
	&is ( $o_locus1->GetSequenceWithEnvironment(10),'GGGGGAAAACCCCGGGGAAACCCGGGAACC',' 4. S+ M Testing GetSequenceWithEnvironment' );
	&is ( $o_locus1->GetSequenceWithEnvironment(15),'CCCCCGGGGGAAAACCCCGGGGAAACCCGGGAACCGGACG','5. S+ E Testing GetSequenceWithEnvironment' );
	&is ( $o_locus1->GetSequenceWithEnvironment(20),'AAAAACCCCCGGGGGAAAACCCCGGGGAAACCCGGGAACCGGACG     ','6. S+ E Testing GetSequenceWithEnvironment' );
	print "\n";
	
	my $o_locus2= New Locus::MiscRegion (start=>10,end=>20,strand=>-1);
	$o_locus2->SetParent($o_annot);
	&is ( $o_locus2->GetSequence(),&Locus::UtilRevComp('GTTAAACCCGG') , '3. S-  Testing GetSequence');
	&is ( $o_locus2->GetSequenceWithEnvironment(5),&Locus::UtilRevComp('AACCGGTTAAACCCGGGTTTA'),'7. S- M Testing GetSequenceWithEnvironment' );
	&is ( $o_locus2->GetSequenceWithEnvironment(10),&Locus::UtilRevComp(' ACGTAACCGGTTAAACCCGGGTTTAAAACC'),'8. S- B Testing GetSequenceWithEnvironment' );
	&is ( $o_locus2->GetSequenceWithEnvironment(15),&Locus::UtilRevComp('      ACGTAACCGGTTAAACCCGGGTTTAAAACCCCGGG'),'9. S- B Testing GetSequenceWithEnvironment' );
	print "\n";
	
	my $o_locus3= New Locus::MiscRegion( start=>321,end=>330,strand=>-1);
	$o_locus3->SetParent($o_annot);
	&is ( $o_locus3->GetSequence(),&Locus::UtilRevComp('CCCGGGGAAA') , '4. S- Testing GetSequence');
	&is ( $o_locus3->GetSequenceWithEnvironment(10),&Locus::UtilRevComp('GGGGGAAAACCCCGGGGAAACCCGGGAACC'),'5. S- M Testing GetSequenceWithEnvironment' );
	&is ( $o_locus3->GetSequenceWithEnvironment(15),&Locus::UtilRevComp('CCCCCGGGGGAAAACCCCGGGGAAACCCGGGAACCGGACG'),'9. S- E Testing GetSequenceWithEnvironment' );
	&is ( $o_locus3->GetSequenceWithEnvironment(20),&Locus::UtilRevComp('AAAAACCCCCGGGGGAAAACCCCGGGGAAACCCGGGAACCGGACG     '),'10. S- E Testing GetSequenceWithEnvironment' );
	print "\n";
}

# generate a sequence
sub GenerateSeq
{
	my $seq;
	my @a_l=qw( A C G T );
	for ( my $i=0;$i<10;$i++)
	{
		for ( my $j=0;$j<4;$j++)
		{
			$seq .= $a_l[$j] x $i;
		}
	}
	for ( my $i=10;$i>0;$i--)
	{
		for ( my $j=0;$j<3;$j++)
		{
			$seq .= $a_l[$j] x $i;
		}
	}
	return $seq;
}

# Create a string for visualizing a sequence
sub Vseq
{
	my ($vseq) = @_;
	my $p=-9;
	$vseq=~s/([ACGT]{10})/$1 /g;
	my @a_vseq = split ' ',$vseq;
	map $_=sprintf ("%3d ",$p+=10).$_, @a_vseq;
	$vseq = join "\n",@a_vseq;
	return $vseq;
}

sub GetGenSeq
{
	my $seq = GenerateSeq();
	return \$seq;
}

